from setuptools import setup

setup(name= 'GaussBin_distribution',
      version= '1.2',
      description= 'Gaussian Distributions & Binomial Distributions',
      packages=['GaussBin_distribution'],
      author= 'Faisal Lawan',
      author_email= 'faisallawan08@gmail.com',
      zip_safe= False)